@@ -34,6 +34,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/types.h>
 #ifndef WIN32
 #include <unistd.h>
+#else
+#include <malloc.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,8 +45,11 @@ static int cond=1;
 
 static const char * capture_card=NULL;
 static const char * playback_card=NULL;
+static const char * camera=NULL;
+static const char *infile=NULL,*outfile=NULL;
 static float ng_threshold=-1;
 static bool_t use_ng=FALSE;
+static bool_t two_windows=FALSE;
 
 /* starting values echo canceller */
 static int ec_len_ms=0, ec_delay_ms=250, ec_framesize;
@@ -141,7 +146,10 @@ const char *usage="mediastream --local <port> --remote <ip:port> --payload <payl
 								"[ --ng (enable noise gate)]\n"
 								"[ --ng-threshold <(float) [0-1]> (noise gate threshold)]\n"
 								"[ --capture-card <name>] \n"
-								"[ --playback-card <name>] \n";
+								"[ --playback-card <name>] \n"
+								"[ --infile	<input wav file>] specify a wav file to be used for input, instead of soundcard\n"
+								"[ --outfile <output wav file>] specify a wav file to write audio into, instead of soundcard\n"
+								"[ --camera <camera id as listed at startup> ]\n";
 
 static void run_media_streams(int localport, const char *remote_ip, int remoteport, int payload, const char *fmtp,
           int jitter, int bitrate, MSVideoSize vs, bool_t ec, bool_t agc, bool_t eq);
@@ -158,6 +166,7 @@ int main(int argc, char * argv[])
 	bool_t ec=FALSE;
 	bool_t agc=FALSE;
 	bool_t eq=FALSE;
+
 	/*create the rtp session */
 	ortp_init();
 	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
@@ -227,6 +236,17 @@ int main(int argc, char * argv[])
 		}else if (strcmp(argv[i],"--ng-threshold")==0){
 			i++;
 			ng_threshold=atof(argv[i]);
+		}else if (strcmp(argv[i],"--two-windows")==0){
+			two_windows=TRUE;
+		}else if (strcmp(argv[i],"--infile")==0){
+			i++;
+			infile=argv[i];
+		}else if (strcmp(argv[i],"--outfile")==0){
+			i++;
+			outfile=argv[i];
+		}else if (strcmp(argv[i],"--camera")==0){
+			i++;
+			camera=argv[i];
 		}
 	}
 
@@ -245,6 +265,7 @@ static void run_media_streams(int localport, const char *remote_ip, int remotepo
 	PayloadType *pt;
 	RtpProfile *profile=rtp_profile_clone_full(&av_profile);
 	OrtpEvQueue *q=ortp_ev_queue_new();	
+	MSWebCam *cam=NULL;
 
 	ms_init();
 	signal(SIGINT,stop_handler);
@@ -267,10 +288,10 @@ static void run_media_streams(int localport, const char *remote_ip, int remotepo
 		audio_stream_enable_noise_gate(audio,use_ng);
 		audio_stream_set_echo_canceller_params(audio,ec_len_ms,ec_delay_ms,ec_framesize);
 		printf("Starting audio stream.\n");
-		audio_stream_start_now(audio,profile,remote_ip,remoteport,remoteport+1,payload,jitter,
-			play,
-			capt,
-			 ec);
+	
+		audio_stream_start_full(audio,profile,remote_ip,remoteport,remoteport+1, payload, jitter,infile,outfile,
+		                        outfile==NULL ? play : NULL ,infile==NULL ? capt : NULL,infile!=NULL ? FALSE: ec);
+		
 		if (audio) {
 			if (use_ng && ng_threshold!=-1)
 				ms_filter_call_method(audio->volsend,MS_VOLUME_SET_NOISE_GATE_THRESHOLD,&ng_threshold);
@@ -285,12 +306,18 @@ static void run_media_streams(int localport, const char *remote_ip, int remotepo
 		printf("Starting video stream.\n");
 		video=video_stream_new(localport, ms_is_ipv6(remote_ip));
 		video_stream_set_sent_video_size(video,vs);
+		video_stream_use_preview_video_window(video,two_windows);
+		
+		if (camera)
+			cam=ms_web_cam_manager_get_cam(ms_web_cam_manager_get(),camera);
+		if (cam==NULL)
+			cam=ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get());
 		video_stream_start(video,profile,
 					remote_ip,
 					remoteport,remoteport+1,
 					payload,
-					jitter,
-					ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get()));
+					jitter,cam
+					);
 		session=video->session;
 #else
 		printf("Error: video support not compiled.\n");
